----------------------------------------------------------------
-- Test Bench pour le composant ALU                           --
-- auteur : Matthias Colin                                    --
-- date   : 17/02/2009                                        --
----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use von_neumann2.pack_Von_Neumann.all;

entity tb_alu is
end tb_alu;

architecture tb_alu_arch of tb_alu is

-- dclaration de l'entit alu
component alu is
port(
	-- les entres
	a : in t_mot; -- 1re oprande
	b : in t_mot; -- 2me oprande
	op : in t_operation; -- opration  excuter sur les oprandes
	enable : in std_logic; -- signal activation de l'alu  niveau bas
	clock : in std_logic; -- signal d'horloge active sur front montant
	reset : in std_logic; -- signal de rinitialisation asynchrone
	
	-- les sorties
	s : out t_mot; -- rsultat
	flag_c : out std_logic; -- flag d'tat sur la retenue
	flag_z : out std_logic 	-- flag d'tat sur un rsultat nul
);
end component;

-- les signaux de branchement de l'ALU
signal a, b, s : t_mot;
signal op : t_operation;
signal enable, clock, reset, flag_c, flag_z : std_logic; 

begin

-- une instance d'ALU
alu1: alu port map(
	a => a,
	b => b,
	op => op,
	enable => enable,
	clock => clock,
	reset => reset,
	s => s,
	flag_c => flag_c,
	flag_z => flag_z
);
	

-- gnration d'un signal d'horloge de priode 20 ns
process
begin
	clock <= '0';
	wait for 10 ns;
	clock <= '1';
	wait for 10 ns;
end process;

-- test de l'ALU : les oprations sont dclenches 1 ns aprs 1 front montant d'horloge
process
begin
	-- initialisation des entres et de l'ALU
	a <= "0000";
	b <= "0000";
	enable <= '1';
	reset <= '0';
	wait for 31 ns;
	-- desactivation du reset;
	reset <= '1';
	-- 2 sries d'oprations (1 avec un cycle de temporisation et 
	-- 1 avec les oprations enchaines)
	for i in 1 downto 0 loop
	    -- 1re opration : 6 + 5 = 11 et pas de retenue
	    a <= "0110";
	    b <= "0101";
	    op <= ADD;
	    enable <= '0';
	    wait for 20 ns;
	    -- 0 puis 1 cycle de repos (1er tour et 2e tour de boucle)
	    enable <= '1';
	    wait for i*20 ns;
	    -- 2me opration : 11 + 7 = 2 et une retenue de 1
	    a <= "1011";
	    b <= "0111";
	    op <= ADD;
	    enable <= '0';
	    wait for 20 ns;
	    -- 0 puis 1 cycle de repos (1er tour et 2e tour de boucle)
	    enable <= '1';
	    wait for i*20 ns;
	    -- 3me opration : 10 + 9 = 3 et une retenue de 1
	    -- on vrifie que la retenue prcdente n'est pas prise en compte
	    a <= "1010";
	    b <= "1001";
	    op <= ADD;
	    enable <= '0';
	    wait for 20 ns;
	    -- 0 puis 1 cycle de repos (1er tour et 2e tour de boucle)
	    enable <= '1';
	    wait for i*20 ns;
	    -- 4me opration : 4 + 3 + retenue prcdente = 8 et pas de retenue
	    a <= "0100";
	    b <= "0011";
	    op <= ADC;
	    enable <= '0';
	    wait for 20 ns;
	    -- 0 puis 1 cycle de repos (1er tour et 2e tour de boucle)
	    enable <= '1';
	    wait for i*20 ns;
	    -- 5me opration : 4 + 3 + retenue prcdente = 7 et pas de retenue
	    a <= "0100";
	    b <= "0011";
	    op <= ADC;
	    enable <= '0';
    	    wait for 20 ns;
	    -- 0 puis 1 cycle de repos (1er tour et 2e tour de boucle)
	    enable <= '1';
	    wait for i*20 ns;
	end loop;
	-- attente de fin de simulation
	wait for 100 ns;
end process;

end tb_alu_arch;

