-- boitier mmoire de 4Kmots de 4bits
-- adresse sur 12 bits
--
-- auteur : Matthias Colin
-- version : 1.0 (7/04/2009)
--
-- Fonctionnement :
-- 
-- les resultats sont disponibles pour le composant
-- client de la mmoire en un temps alatoire (dfaut ou pas de cache)
--
-- le composant peut lire ou crire des donnes en rafale (1/cycle)  trois conditions:
--	1. le type d'operation ne change pas (ecriture ou lecture)
--	2. les adresses sont conscutives en mmoire
--	3. il n'y a pas de dfaut de cache (mmoire ready)
-- dans ce cas il suffit de maintenir le signal enable a true pdt la dure de l'opration
-- dans le cas contraire, il faut respecter 1 cycle de repos entre 2 oprations
-- dans le cas d'une mauvaise utilisation la mmoire se bloque et il faut faire un reset

library ieee;
use ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

use vonneuman2010.pack_Von_Neumann.all;

entity memory_with_cache is
port (
	-- control --
	clock : in std_logic; -- rising edge
	reset : in std_logic; -- active at lowlevel
	-- interface CPU --
	data_in : in t_mot;
	data_out : out t_mot;
	address : in t_address12;
	rw : in std_logic; -- 1 : read / 0 : write
	cs : in std_logic; -- memory activation (low level)
	ready : out boolean -- cache is ready to be used / to be read
);
end memory_with_cache;

architecture memory_with_cache_proto_arch of memory_with_cache is
-- les tats de la mmoire
-- RDY : mmoire prte
-- W : mmoire en cours d'ecriture
-- R : mmoire en cours de lecture
-- ERR : mmoire bloque 
type t_etat is (RDY, W, R, ERR, AttenteR, AttenteW);
signal etat : t_etat;
	
-- le tableau des mots en mmoire
type t_memory is array(0 to 2**12-1) of t_mot;
signal mots : t_memory;

begin
	
	process(clock, reset) is
		-- index pour parcourir la memoire
		variable i : integer range 0 to 2**12-1; 
		-- dernire case accde (pour les oprations en rafales)
		variable oldIndice : integer range 0 to 2**12-1;
		-- buffer pour stocker de manire provisoire la donne  crire
		variable copie_data_in : t_mot;
		-- compteur de cycle d'horloge
		variable cpt : integer range 0 to 15;
		constant DUREE_TRAITEMENT_DEFAUT : integer range 0 to 15 := 15;
		-- pour simuler des dfauts de cache avec un taux de 25 %
		variable randomDefaut : integer range 0 to 15;
	begin
		if reset='0' then
			-- initialisation de la memoire
			
			-- chargement du programme de la factorielle 3 en dur
			mots(0) <= "0100";	-- 		LD R0 3
			mots(1) <= "0011";
			
			mots(2) <= "0111";	-- 		LD R3 R0

			mots(3) <= "0100"; 	-- 		LD R0 1
			mots(4) <= "0001";	
			
			mots(5) <= "1110";	-- bcle : 	MULT R0 R3
			mots(6) <= "1111";
			
			mots(7) <= "1101";	-- 		DJNZ bcle (dep = -6)
			mots(8) <= "1111";
			mots(9) <= "1111";
			mots(10) <= "1010"; 

			mots(11) <= "1100"; 	-- halt : 	JR halt (dep = -4)
			mots(12) <= "1111";
			mots(13) <= "1111";
			mots(14) <= "1100";
			
			-- tous les autres mots sont remis a zero
			FOR i IN 15 to 2**12-1 LOOP
				mots(i) <= (others => '0');
			END LOOP;
			-- la mmoire est prte  tre lue ou crite
			etat <= RDY;
			ready <= true;
			randomDefaut := 0;
		elsif clock'event and clock='1' then
			-- traitement d'un front montant de l'horloge
			if (cs = '0') then
				-- botier mmoire actif
				case etat is
					-- ETAT PRET INITIAL --
					when RDY =>
						i := conv_integer(address);
						-- sauvegarde de l'indice pour
						-- tester les lectures/ecritures en rafale
						oldIndice := i;
						randomDefaut := (randomDefaut + 1) mod 3;
						if rw = '1' then
						-- lecture --
							if randomDefaut = 2 then
								cpt := 0;
								etat <= attenteR;
								ready <= false;
							else
								data_out <= mots(i);
								etat <= R;
								ready <= true;
							end if;
						else
						-- ecriture --
							if randomDefaut = 2 then
								cpt := 0;
								etat <= attenteW;
								ready <= false;
								copie_data_in := data_in;
							else
								mots(i) <= data_in;
								etat <= W;
								ready <= true;
							end if;
						end if;
					-- ETAT ECRITURES EN RAFALES --	
					when W =>
						-- criture en rafale
						i := conv_integer(address);
						-- vrifier adresses conscutives
						if (i /= (oldIndice+1)) then
							etat <= ERR;
						elsif rw = '1' then
						-- lecture : impossible juste aprs une criture --
							etat <= ERR;
						else
						-- criture --
							randomDefaut := (randomDefaut + 1) mod 3;
							if randomDefaut = 2 then
								cpt := 0;
								etat <= attenteW;
								copie_data_in := data_in;
								ready <= false;
							else
								mots(i) <= data_in;
								oldIndice := i;
								ready <= true;
							end if;
						end if;
					-- ETAT LECTURES EN RAFALES --	
					when R =>
						-- ecriture en rafale
						i := conv_integer(address);
						-- vrifier address consecutive
						if (i /= (oldIndice+1)) then
							etat <= ERR;
						elsif rw = '1' then
						-- lecture --
							randomDefaut := (randomDefaut + 1) mod 3;
							if randomDefaut = 2 then
								cpt := 0;
								etat <= attenteR;
								ready <= false;
							else
								data_out <= mots(i);
								oldIndice := i;
								ready <= true;
							end if;
						else
						-- criture : impossible juste aprs une lecture --
							etat <= ERR;
						end if;
					-- ETAT LECTURE DIFFEREE AVEC DEFAUT DE CACHE (SIMULE) --
					-- ne peut pas recevoir d'ordre si en attente
					when attenteR =>
						etat <= ERR;
					-- ETAT ECRITURE DIFFEREE AVEC DEFAUT DE CACHE (SIMULE) --
					-- ne peut pas recevoir d'ordre si en attente
					when attenteW =>
						etat <= ERR;
					-- ETAT ERREUR --	
					when ERR =>
						-- etat terminal : memoire plantee
						null;
				end case;
			else -- cs = '1'
			-- botier dsactiv
				case etat is
					-- ETAT LECTURE DIFFEREE AVEC DEFAUT DE CACHE (SIMULE) --
					when attenteR =>
						if cpt < DUREE_TRAITEMENT_DEFAUT then
							cpt := cpt + 1;
						else
							-- lecture
							data_out <= mots(i);
							oldIndice := i;
							etat <= R;
							ready <= true;
						end if;
					-- ETAT ECRITURE DIFFEREE AVEC DEFAUT DE CACHE (SIMULE) --
					when attenteW =>
						if cpt < DUREE_TRAITEMENT_DEFAUT then
							cpt := cpt + 1;
						else
							-- ecriture
							mots(i) <= copie_data_in;
							oldIndice := i;
							etat <= W;
							ready <= true;
						end if;
				--	si en fonctionnement retour en mode prt
					when W =>
						etat <= RDY;
						ready <= true;					
					when R =>
						etat <= RDY;
						ready <= true;
				--	sinon reste en l'tat (prt ou erreur)
					when others => null;

				end case;
			end if;
		end if;
	end process;
	
end memory_with_cache_proto_arch;	






