----------------------------------------------------------------
-- Test Bench pour le composant memory4Kx4bits                --
-- auteur : Matthias Colin                                    --
-- date   : 17/02/2009                                        --
----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

use von_neumann2.pack_Von_Neumann.all;

entity tb_memory_with_cache is
end tb_memory_with_cache;

architecture tb_memory_with_cache_arch of tb_memory_with_cache is

component memory_with_cache is
port (
	-- control --
	clock : in std_logic; -- rising edge
	reset : in std_logic; -- active at lowlevel
	-- interface CPU --
	data_in : in t_mot;
	data_out : out t_mot;
	address : in t_address12;
	rw : in std_logic; -- 1 : read / 0 : write
	cs : in std_logic; -- memory activation (low level)
	ready : out boolean -- cache is ready to be used / to be read
);
end component;

signal tb_clock, tb_reset, tb_rw, tb_cs : std_logic;
signal tb_data_in, tb_data_out : t_mot;
signal tb_address : t_address12;
signal tb_ready : boolean;

begin

-- une instance du composant mémoire
u_memory : memory_with_cache port map (
	clock => tb_clock, reset => tb_reset, data_in => tb_data_in, data_out => tb_data_out, 
		address => tb_address, rw => tb_rw, cs => tb_cs, ready => tb_ready);

-- génération d'un signal d'horloge de période 20 ns
process
begin
	tb_clock <= '0';
	wait for 10 ns;
	tb_clock <= '1';
	wait for 10 ns;
end process;

-- test du composant mémoire
-- NB : pour débugger, réduire les boucles pour n'utliser qu'une partie de la mémoire
process
begin
	-- initialisation
	tb_reset <= '0';
	tb_cs <= '1';
	tb_address <= (others => 'X');
	tb_data_in <= (others => 'X');
	tb_rw <= 'X';
	wait for 20 ns;
	tb_reset <= '1';
	
	wait for 11 ns;
	-- 1er top après initialisation
	
	-- test 1 : écriture en rafale de toute la mémoire
--	for i in 0 to 4095 loop
	for i in 0 to 10 loop
		tb_cs <= '0';
		tb_rw <= '0';
		tb_address <= conv_std_logic_vector(i,12);
		tb_data_in <= conv_std_logic_vector(i,4); -- on écrit les 4 bits de pds faible de l'@
		wait for 20 ns;
		while (not(tb_ready)) loop
			tb_cs <= '1';
			wait for 20 ns;
		end loop;
	end loop;
	
	-- 1 cycle de repos
	tb_cs <= '1';
	wait for 20 ns;
	
	-- test 2 : lecture en rafale de toute la mémoire
	-- résultat à lire dans tb_data_out
--	for i in 0 to 4095 loop
	for i in 0 to 10 loop
		tb_cs <= '0';
		tb_rw <= '1';
		tb_address <= conv_std_logic_vector(i,12);
		wait for 20 ns;
		while (not(tb_ready)) loop
			tb_cs <= '1';
			wait for 20 ns;
		end loop;
	end loop;
	
	-- 1 cycle de repos
	tb_cs <= '1';
	wait for 20 ns;

	
	-- test 3 : écriture puis lecture de ttes les données possibles dans toutes les cases mémoires
--	for i in 0 to 4095 loop
	for i in 0 to 10 loop	
		for j in 0 to 15 loop
			-- 1 - ECRITURE
			-- adresse
			tb_address <= conv_std_logic_vector(i, 12);
			-- donnée
			tb_data_in <= conv_std_logic_vector(j, 4);
			-- ordre écriture
			tb_cs <= '0';
			tb_rw <= '0';
			wait for 20 ns;
			tb_cs <= '1';
			while (not(tb_ready)) loop
				wait for 20 ns;
			end loop;
		
			-- 2 - REPOS
			-- 1 cycle de repos
			tb_cs <= '1';
			wait for 20 ns;
			
			-- 3 - LECTURE
			-- même adresse (i sur 12 bits)
			-- ordre lecture :
			tb_cs <= '0';
			tb_rw <= '1';
			wait for 20 ns;
			tb_cs <= '1';
			while (not(tb_ready)) loop
				wait for 20 ns;
			end loop;
			
			-- 4 - REPOS
			-- 1 cycle de repos
			tb_cs <= '1';
			wait for 20 ns;
		end loop;
	end loop;
	
	-- désactivation
	tb_cs <= '1';
	wait for 200 ns;
end process;

end tb_memory_with_cache_arch;






