------------------------------------------
-- Composant Machine de Von Neumann     --
-- assemblage final 			--
-- auteur : Matthias Colin		--
-- date   : 23/06/2009           	--
--                              	--
------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use vonneuman2010.pack_Von_Neumann.all;
use vonneuman2010.pack_cpu.all;

entity von_neumann is
end von_neumann;

architecture von_neumann_arch of von_neumann is

component cpu is
port (
	-- control --
	clock : in std_logic; -- rising edge
	reset : in std_logic; -- active at lowlevel
	
	-- interface MEM --
	mdr_in : in t_mot;
	mdr_out : out t_mot;
	mar : out t_address12;
	rw_mem : out std_logic; -- 1 : read / 0 : write
	cs_mem : out std_logic; -- memory activation (low level)
	ready_mem : in boolean -- cache is ready to be used / to be read
	
	-- interface ALU --
	-- next time
);
end component;


-- component alu : next time

component memory_with_cache is
port (
	-- control --
	clock : in std_logic; -- rising edge
	reset : in std_logic; -- active at lowlevel
	-- interface CPU --
	data_in : in t_mot;
	data_out : out t_mot;
	address : in t_address12;
	rw : in std_logic; -- 1 : read / 0 : write
	cs : in std_logic; -- memory activation (low level)
	ready : out boolean -- cache is ready to be used / to be read
);
end component;

-- signaux de connexions --
signal	clock, reset : std_logic;
signal 	data_mem_to_cpu, data_cpu_to_mem : t_mot;
signal 	address : t_address12;
signal  rw_mem, cs_mem : std_logic;
signal 	ready_mem : boolean;

begin
	-- instances de composants --

	-- le CPU --
	cpu_vn : cpu port map (
		clock => clock, reset => reset, 
		mdr_in => data_mem_to_cpu,
		mdr_out => data_cpu_to_mem,
		mar => address,
		rw_mem => rw_mem, cs_mem => cs_mem, ready_mem => ready_mem
		);

	-- l'ALU --
	-- next time
	
	-- la Mmoire --
	mem_vn : memory_with_cache port map(
		clock => clock, reset => reset, 
		data_in => data_cpu_to_mem,
		data_out => data_mem_to_cpu,
		address => address,
		rw => rw_mem, cs => cs_mem, ready => ready_mem);

	-- process d'horloge
	process
	begin
		clock <= '0';
		wait for 10 ns;
		clock <= '1';
		wait for 10 ns;
	end process;
	
	-- process d'initialisation de la machine
	-- NB : on pourrait externaliser ce seul signal
	process
	begin
		-- initialisation des 3 composants
		-- (le programme est en dur dans la RAM)
		reset <= '0';
		wait for 35 ns;
		reset <= '1';
		-- attente 1000 cycles
		wait for 1000*20 ns;
	end process;

end von_neumann_arch;


